@@ -33,18 +33,18 @@
 
 (sb!xc:deftype signed-byte (&optional s)
   (cond ((eq s '*) 'integer)
-	((and (integerp s) (> s 0))
-	 (let ((bound (ash 1 (1- s))))
-	   `(integer ,(- bound) ,(1- bound))))
-	(t
-	 (error "bad size specified for SIGNED-BYTE type specifier: ~S" s))))
+        ((and (integerp s) (> s 0))
+         (let ((bound (ash 1 (1- s))))
+           `(integer ,(- bound) ,(1- bound))))
+        (t
+         (error "bad size specified for SIGNED-BYTE type specifier: ~S" s))))
 
 (sb!xc:deftype unsigned-byte (&optional s)
   (cond ((eq s '*) '(integer 0))
-	((and (integerp s) (> s 0))
-	 `(integer 0 ,(1- (ash 1 s))))
-	(t
-	 (error "bad size specified for UNSIGNED-BYTE type specifier: ~S" s))))
+        ((and (integerp s) (> s 0))
+         `(integer 0 ,(1- (ash 1 s))))
+        (t
+         (error "bad size specified for UNSIGNED-BYTE type specifier: ~S" s))))
 
 ;;; ANSI got UNSIGNED-BYTE wrong, prohibiting (UNSIGNED-BYTE 0).
 ;;; Since this is actually a substantial impediment to clarity...
@@ -56,10 +56,11 @@
 
 (sb!xc:deftype bit () '(integer 0 1))
 
-(sb!xc:deftype compiled-function () 'function)
-
 (sb!xc:deftype atom () '(not cons))
 
+(sb!xc:deftype base-char ()
+  '(character-set ((0 . #.(1- base-char-code-limit)))))
+
 (sb!xc:deftype extended-char ()
   #!+sb-doc
   "Type of CHARACTERs that aren't BASE-CHARs."
@@ -69,9 +70,9 @@
   #!+sb-doc
   "Type corresponding to the characters required by the standard."
   '(member
-    #\NEWLINE #\SPACE #\! #\" #\# #\$ #\% #\& #\' #\( #\) #\* #\+ #\,
+    #\Newline #\Space #\! #\" #\# #\$ #\% #\& #\' #\( #\) #\* #\+ #\,
     #\- #\. #\/ #\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9 #\: #\; #\< #\=
-    #\> #\?  #\@ #\A #\B #\C #\D #\E #\F #\G #\H #\I #\J #\K #\L #\M
+    #\> #\? #\@ #\A #\B #\C #\D #\E #\F #\G #\H #\I #\J #\K #\L #\M
     #\N #\O #\P #\Q #\R #\S #\T #\U #\V #\W #\X #\Y #\Z #\[ #\\ #\]
     #\^ #\_ #\` #\a #\b #\c #\d #\e #\f #\g #\h #\i #\j #\k #\l #\m
     #\n #\o #\p #\q #\r #\s #\t #\u #\v #\w #\x #\y #\z #\{
@@ -129,9 +130,7 @@
 ;;; FIXME: could probably go away
 (sb!xc:deftype form () t)
 
-;;; Maclisp compatibility...
-;;; FIXME: should be STRING-DESIGNATOR (the term used in the ANSI spec)
-(sb!xc:deftype stringable () '(or string symbol character))
+(sb!xc:deftype string-designator () '(or string symbol character))
 
 ;;; a thing legal in places where we want the name of a file
 (sb!xc:deftype filename () '(or string pathname))
@@ -142,6 +141,10 @@
 (sb!xc:deftype logical-host-designator ()
   '(or host string))
 
+(sb!xc:deftype package-designator () '(or string-designator sb!xc:package))
+;;; a designator for a list of symbols
+(sb!xc:deftype symbols-designator () '(or list symbol))
+
 ;;; a thing returned by the irrational functions. We assume that they
 ;;; never compute a rational result.
 (sb!xc:deftype irrational ()
@@ -151,7 +154,8 @@
 (sb!xc:deftype char-code () `(integer 0 (,sb!xc:char-code-limit)))
 
 ;;; a consed sequence result. If a vector, is a simple array.
-(sb!xc:deftype consed-sequence () '(or list (simple-array * (*))))
+(sb!xc:deftype consed-sequence ()
+  '(or (simple-array * (*)) (and sequence (not vector))))
 
 ;;; the :END arg to a sequence
 (sb!xc:deftype sequence-end () '(or null index))
@@ -161,10 +165,7 @@
   `(or null integer))
 
 ;;; a valid argument to a stream function
-;;;
-;;; FIXME: should probably be STREAM-DESIGNATOR, after the term
-;;; used in the ANSI spec (if this is in fact exactly the same thing)
-(sb!xc:deftype streamlike () '(or stream (member nil t)))
+(sb!xc:deftype stream-designator () '(or stream (member nil t)))
 
 ;;; an object suitable for input to standard functions that accept
 ;;; "environment objects" (of the ANSI glossary)
@@ -178,28 +179,28 @@
 ;;; decomposing floats into integers
 (sb!xc:deftype single-float-exponent ()
   `(integer ,(- sb!vm:single-float-normal-exponent-min
-		sb!vm:single-float-bias
-		sb!vm:single-float-digits)
-	    ,(- sb!vm:single-float-normal-exponent-max
-		sb!vm:single-float-bias)))
+                sb!vm:single-float-bias
+                sb!vm:single-float-digits)
+            ,(- sb!vm:single-float-normal-exponent-max
+                sb!vm:single-float-bias)))
 (sb!xc:deftype double-float-exponent ()
   `(integer ,(- sb!vm:double-float-normal-exponent-min
-		sb!vm:double-float-bias
-		sb!vm:double-float-digits)
-	    ,(- sb!vm:double-float-normal-exponent-max
-		sb!vm:double-float-bias)))
+                sb!vm:double-float-bias
+                sb!vm:double-float-digits)
+            ,(- sb!vm:double-float-normal-exponent-max
+                sb!vm:double-float-bias)))
 (sb!xc:deftype single-float-int-exponent ()
   `(integer ,(- sb!vm:single-float-normal-exponent-min
-		sb!vm:single-float-bias
-		(* sb!vm:single-float-digits 2))
-	    ,(- sb!vm:single-float-normal-exponent-max
-		sb!vm:single-float-bias
-		sb!vm:single-float-digits)))
+                sb!vm:single-float-bias
+                (* sb!vm:single-float-digits 2))
+            ,(- sb!vm:single-float-normal-exponent-max
+                sb!vm:single-float-bias
+                sb!vm:single-float-digits)))
 (sb!xc:deftype double-float-int-exponent ()
   `(integer ,(- sb!vm:double-float-normal-exponent-min sb!vm:double-float-bias
-		(* sb!vm:double-float-digits 2))
-	    ,(- sb!vm:double-float-normal-exponent-max sb!vm:double-float-bias
-		sb!vm:double-float-digits)))
+                (* sb!vm:double-float-digits 2))
+            ,(- sb!vm:double-float-normal-exponent-max sb!vm:double-float-bias
+                sb!vm:double-float-digits)))
 (sb!xc:deftype single-float-significand ()
   `(integer 0 (,(ash 1 sb!vm:single-float-digits))))
 (sb!xc:deftype double-float-significand ()
