@@ -16,9 +16,20 @@
 #ifndef _GC_H_
 #define _GC_H_
 
+#include "sbcl.h"
+
+#ifdef LISP_FEATURE_GENCGC
+#define PAGE_BYTES GENCGC_PAGE_SIZE
+#else
+#define PAGE_BYTES 0x1000
+#endif
+
+typedef signed long page_index_t;
+typedef signed char generation_index_t;
+
 extern void gc_init(void);
 extern void gc_initialize_pointers(void);
-extern void collect_garbage(unsigned last_gen);
+extern void collect_garbage(generation_index_t last_gen);
 extern void gc_init_tables(void);
 
 
@@ -28,5 +39,9 @@ extern void set_auto_gc_trigger(os_vm_size_t usage);
 extern void clear_auto_gc_trigger(void);
 
 extern int maybe_gc_pending;
-extern int gc_thread_pid;
+
+#include "fixnump.h"
+
+#include "pseudo-atomic.h"
+
 #endif /* _GC_H_ */
